@@ -4,11 +4,11 @@
 int main(int argc, char **argv)
 {   
 	// Creating a filled vector
-        unsigned int i = 0;
+    unsigned int i = 0;
 	const char *vector[] = {"aaaaaaaaaa", "bbbbbbbbbb", "cccccccccc", "dddddddddd", "eeeeeeeeee", 
-				"ffffffffff", "gggggggggg", "hhhhhhhhhh", "iiiiiiiiii", "jjjjjjjjjj"};
+							"ffffffffff", "gggggggggg", "hhhhhhhhhh", "iiiiiiiiii", "jjjjjjjjjj"};
 	unsigned int nkeys = 10;
-        FILE* mphf_fd = fopen("temp.mph", "w");
+    FILE* mphf_fd = fopen("temp.mph", "w");
 	// Source of keys
 	cmph_io_adapter_t *source = cmph_io_vector_adapter((char **)vector, nkeys);
 
@@ -18,23 +18,23 @@ int main(int argc, char **argv)
 	cmph_config_set_mphf_fd(config, mphf_fd);
 	cmph_t *hash = cmph_new(config);
 	cmph_config_destroy(config);
-        cmph_dump(hash, mphf_fd); 
+    cmph_dump(hash, mphf_fd); 
 	cmph_destroy(hash);	
 	fclose(mphf_fd);
    
 	//Find key
-        mphf_fd = fopen("temp.mph", "r");
-        hash = cmph_load(mphf_fd);
-        while (i < nkeys) {
-           const char *key = vector[i];
-	   unsigned int id = cmph_search(hash, key, strlen(key));
+    mphf_fd = fopen("temp.mph", "r");
+    hash = cmph_load(mphf_fd);
+    while (i < nkeys) {
+       const char *key = vector[i];
+	   unsigned int id = cmph_search(hash, key, (cmph_uint32)strlen(key));
 	   fprintf(stderr, "key:%s -- hash:%u\n", key, id);
-           i++;
-        }
-        
+       i++;
+    }
+    
 	//Destroy hash
 	cmph_destroy(hash);
 	cmph_io_vector_adapter_destroy(source);   
-        fclose(mphf_fd);
+    fclose(mphf_fd);
 	return 0;
 }
