@@ -10,11 +10,12 @@ struct __vqueue_t
 
 vqueue_t * vqueue_new(cmph_uint32 capacity)
 {
+  size_t capacity_plus_one = capacity + 1;
   vqueue_t *q = (vqueue_t *)malloc(sizeof(vqueue_t));
   assert(q);
-  q->values = (cmph_uint32 *)calloc(capacity+1, sizeof(cmph_uint32));
+  q->values = (cmph_uint32 *)calloc(capacity_plus_one, sizeof(cmph_uint32));
   q->beg = q->end = 0;
-  q->capacity = capacity+1;
+  q->capacity = capacity_plus_one;
   return q;
 }
 
